@@ -3,23 +3,21 @@
 
 distname = "apycotbot"
 modname = "apycotbot"
-
-numversion = (1, 6, 0)
-version = '.'.join(str(num) for num in numversion)
-
 license = 'GPL'
-copyright = '''Copyright (c) 2003-2009 LOGILAB S.A. (Paris, FRANCE).
-http://www.logilab.fr/ -- mailto:contact@logilab.fr'''
+
+numversion = (1, 9, 0)
+version = '.'.join(str(num) for num in numversion)
 
 author = "Logilab"
 author_email = "contact@logilab.fr"
+description = "Apycot bot client dedicated to launch test suites"
+web = 'http://www.logilab.org/project/apycot'
 
-short_desc = "Apycot bot client dedicated to launch test suites"
-long_desc = """This version of apycot can now use the cubicweb framework."""
+install_requires = ['cubicweb-apycot >= %s' % version,
+                    'logilab-common >= 0.50.0',
+                    'logilab-devtools',
+                    ]
 
-web = ''
-ftp = ''
-pyversions = ['2.4']
 
 from os.path import join
 scripts = (join('bin', 'apycotbot'),
