@@ -1,7 +1,7 @@
 """checkers for python source files
 
 :organization: Logilab
-:copyright: 2003-2009 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 :license: General Public License version 2 - http://www.gnu.org/licenses
 """
@@ -21,6 +21,7 @@ from logilab.common.fileutils import norm_read
 from logilab.common.textutils import splitstrip
 from logilab.common.decorators import cached
 from logilab.common.compat import any
+from logilab.common.proc import RESOURCE_LIMIT_EXCEPTION
 
 try:
     from logilab.devtools.lib.pkginfo import PackageInfo
@@ -83,7 +84,7 @@ class PythonSyntaxChecker(AbstractFilteredFileChecker):
     """check syntax of python file
 
        Use Pylint to check a score for python package. The check fails if the score is
-       inferior to a given treshold.
+       inferior to a given threshold.
     """
 
     id = 'python_syntax'
@@ -107,6 +108,7 @@ register('checker', PythonSyntaxChecker)
 
 
 class PyTestParser(SimpleOutputParser):
+    status = NODATA
     non_zero_status_code = FAILURE
     # search for following output:
     #
@@ -124,23 +126,33 @@ class PyTestParser(SimpleOutputParser):
 
     total, failures, errors, skipped = 0, 0, 0, 0
 
+    def __init__(self, writer, options=None):
+        super(PyTestParser, self).__init__(writer, options)
+        self.total    = 0
+        self.failures = 0
+        self.skipped  = 0
+        self.errors   = 0
+        #self.success is dynamiclty computed
+        #self.status = SUCCESS
+
     def _parse(self, stream):
+        self.status = None
         super(PyTestParser, self)._parse(stream)
-        if self.total == 0:
-            self.writer.error('File doesn\'t contains any test cases',
-                              path=self.path)
-            self.set_status(NODATA)
-        elif self.errors or self.failures:
+        if self.errors or self.failures:
             self.set_status(FAILURE)
         elif self.skipped:
             self.set_status(PARTIAL)
+        elif not self.total:
+            self.set_status(NODATA)
+        elif self.total >= 0:
+            self.set_status(SUCCESS)
 
     @property
     def success(self):
         return max(0, self.total - sum(( self.failures, self.errors,
                                          self.skipped,)))
     def add_junk(self, line):
-        if any(c for c in line if c not in 'EF. \n\t\r-*'):
+        if any(c for c in line if c not in 'EFS. \n\t\r-*'):
             self.unparsed.append(line)
 
     def extract_tests_status(self, values):
@@ -155,10 +167,11 @@ class PyTestParser(SimpleOutputParser):
         match = self.regex.match(line)
         if match is not None:
             values = match.groupdict()
-            self.total = int(values['total'])
+            total = int(values['total'])
+            self.total += total
             self.extract_tests_status(values)
-            return
-        self.add_junk(line)
+        else:
+            self.add_junk(line)
 
 
 class PyTestChecker(BaseChecker):
@@ -244,7 +257,7 @@ class PyTestChecker(BaseChecker):
     def coverage_enabled(self):
         return coverage is not None and self._coverage is not None
 
-    def _run(self, test):
+    def setup_check(self, test):
         """run the checker against <path> (usually a directory)"""
         test_support.verbose = 0
         self.test = test
@@ -257,21 +270,21 @@ class PyTestChecker(BaseChecker):
                 break
         else:
             self._coverage = None
-        return self._start_unittest()
+        return SUCCESS
 
-    def _start_unittest(self):
+    def do_check(self, test):
         if self.coverage_enabled():
-            self._coverage.coverage_data_dir = pyinstall_path(self.test)
+            self._coverage.coverage_data_dir = pyinstall_path(test)
             command = ['-c', 'from logilab.common.pytest import run; import sys; sys.argv=["pytest", "--coverage"]; run()']
         else:
             command = ['-c', 'from logilab.common.pytest import run; run()']
         extraargs = self.get_option("argument", [])
         if not isinstance(extraargs, list):
-            command.apend(extraargs)
+            command.append(extraargs)
         else:
             command += extraargs
         cwd = os.getcwd()
-        os.chdir(pyinstall_path(self.test))
+        os.chdir(pyinstall_path(test))
         try:
             status = SUCCESS
             testresults = {'success': 0, 'failures': 0,
@@ -311,6 +324,7 @@ class PyTestChecker(BaseChecker):
                             path=self._path)
         #cmd.parser.path = join(self._path, command[0]) # XXX
         cmd.run()
+        cmd.set_status(cmd.parser.status)
         return cmd
 
 register('checker', PyTestChecker)
@@ -367,16 +381,16 @@ class PyUnitTestChecker(PyTestChecker):
          },
         ]
 
-    def _start_unittest(self):
+    def do_check(self, test):
         status = SUCCESS
         testdirs = splitstrip(self.get_option("test_dirs", 'test,tests'))
-        basepath = self.test.project_path(subpath=True)
+        basepath = test.project_path(subpath=True)
         for testdir in testdirs:
             testdir = join(basepath, testdir)
             if exists(testdir):
                 self._path = testdir
                 if self._coverage is not None:
-                    self._coverage.coverage_data_dir = pyinstall_path(self.test)
+                    self._coverage.coverage_data_dir = pyinstall_path(test)
                 # FIXME keep compatibility with python2.4 in Debian stable (etch)
                 # FIXME Feel free to use the 'with' statement afterwards.
                 #with pushd(testdir):
@@ -403,13 +417,13 @@ class PyUnitTestChecker(PyTestChecker):
             return NODATA
         status = SUCCESS
         testresults = {'success': 0, 'failures': 0,
-                      'errors': 0, 'skipped': 0}
+                       'errors': 0, 'skipped': 0}
         total = 0
         for python in self.pyversions:
             for test_file in tests:
                 cmd = self.run_test(test_file + '.py', python)
+                total += cmd.parser.total
                 for rtype in testresults:
-                    total += getattr(cmd.parser, rtype)
                     testresults[rtype] += getattr(cmd.parser, rtype)
                 status = self.merge_status(status, cmd.status)
         self.execution_info(total, testresults)
@@ -428,16 +442,29 @@ register('checker', PyUnitTestChecker)
 class PyDotTestParser(PyUnitTestParser):
     line_regex = re.compile(
             r'(?P<filename>\w+\.py)(\[(?P<ntests>\d+)\] | - )(?P<results>.*)')
+
+    # XXX overwrite property
     success = 0
+
     def _parse(self, stream):
         for testname, _, _, results in self.line_regex.findall(stream.read()):
             if results == "FAILED TO LOAD MODULE":
                 self.errors += 1
             else:
                 self.success += results.count('.')
+                self.total += results.count('.')
                 self.failures += results.count('F')
+                self.total += results.count('F')
                 self.errors += results.count('E')
+                self.total += results.count('E')
                 self.skipped += results.count('s')
+                self.total += results.count('s')
+        if self.failures or self.errors:
+            self.set_status(FAILURE)
+        elif self.skipped:
+            self.set_status(PARTIAL)
+        elif not self.success:
+            self.set_status(NODATA)
 
 
 class PyDotTestChecker(PyUnitTestChecker):
@@ -463,16 +490,20 @@ class PyLintChecker(BaseChecker):
     need_preprocessor = 'install'
     id = 'pylint'
     options_def = [
-        {'name': 'threshold', 'type': 'int',
+        {'name': 'threshold',
+         'type': 'int',
          'help': ('integer between 1 and 10 telling expected pylint note to '
                   'pass this check. Default to 7.'),
+         'default': 7,
          },
         {'name': 'pylintrc',
          'help': ('path to a pylint configuration file.'),
          },
-        {'name': 'show_categories', 'type': 'csv',
+        {'name': 'show_categories',
+         'type': 'csv',
          'help': ('comma separated list of pylint message categories to add to '
                   'reports. Default to error (E) and failure (F).'),
+         'default': 'E,F'
          },
         {'name': 'additional_builtins',
          'help': ('comma separated list of additional builtins to give to '
@@ -487,14 +518,11 @@ class PyLintChecker(BaseChecker):
     def version_info(self):
         self.record_version_info('pylint', pylint_version)
 
-    def _run(self, test):
+    def do_check(self, test):
         """run the checker against <path> (usually a directory)"""
-        threshold = int(self.get_option('threshold', 7))
+        threshold = int(self.get_option('threshold'))
         pylintrc_path = self.get_option('pylintrc', None)
         linter = PyLinter(pylintrc=pylintrc_path)
-        linter.set_option('persistent', False)
-        linter.set_option('reports', 0, action='store')
-        linter.quiet = 1
         # register checkers
         checkers.initialize(linter)
         # load configuration
@@ -503,6 +531,9 @@ class PyLintChecker(BaseChecker):
             linter.load_file_configuration(join(package_wd_path, 'pylintrc'))
         else:
             linter.load_file_configuration()
+        linter.set_option('persistent', False)
+        linter.set_option('reports', 0, action='store')
+        linter.quiet = 1
         # set file or dir to ignore
         for option in ('ignore', 'additional_builtins', 'disable_msg'):
             try:
@@ -512,7 +543,7 @@ class PyLintChecker(BaseChecker):
             if value is not None:
                 linter.global_set_option(option.replace('_', '-'), ','.join(value))
         # message categories to record
-        categories = self._csv_option('show_categories', ('E', 'F'))
+        categories = self._csv_option('show_categories')
         linter.set_reporter(MyLintReporter(self.writer, test.tmpdir, categories))
         # run pylint
         linter.check(pyinstall_path(test))
@@ -595,7 +626,7 @@ class PyCoverageChecker(BaseChecker):
 
     When devtools is available, test will be launched in a coverage mode. This
     test will gather coverage information, and will succeed if the test coverage
-    is superior to a given treshold. *This checker must be executed after the
+    is superior to a given threshold. *This checker must be executed after the
     python_unittest checker.
     """
 
@@ -604,6 +635,7 @@ class PyCoverageChecker(BaseChecker):
         {'name': 'threshold', 'type': 'int',
          'help': ('integer between 1 and 100 telling expected percent coverage '
                   'to pass this check. Default to 80.'),
+         'default': 80,
          },
         ]
     coverage_data_dir = None # set by the checker generating coverage data
@@ -611,10 +643,10 @@ class PyCoverageChecker(BaseChecker):
     def version_info(self):
         self.record_version_info('devtools', devtools_version)
 
-    def _run(self, test):
+    def do_check(self, test):
         """run the checker against <path> (usually a directory)"""
-        threshold = int(self.get_option('threshold', 80))
-        assert threshold, 'no threshold defined'
+        self.threshold = int(self.get_option('threshold'))
+        assert self.threshold, 'no threshold defined'
         # XXX check the pyunit check is executed before (with coverage
         # activated) ?
         if os.environ.get('COVERAGE_FILE'):
@@ -630,13 +662,13 @@ class PyCoverageChecker(BaseChecker):
             os.chdir(self.coverage_data_dir)
             try:
                 if not exists('.coverage'):
-                    self.writer.error('no coverage information',
+                    self.writer.fatal('no coverage information',
                                       path=self.coverage_data_dir)
                     return NODATA
                 percent = self._get_cover_info(pyinstall_path(test))
             finally:
                 os.chdir(cwd)
-        if percent < threshold:
+        if percent < self.threshold:
             return FAILURE
         return SUCCESS
 
@@ -657,7 +689,12 @@ class PyCoverageChecker(BaseChecker):
         result.sort()
         for _, name, pc_cover, readable in result:
             msg = '%d %% covered, missing %s' % (pc_cover, readable)
-            self.writer.info(msg, path=name)
+            if pc_cover < ( self.threshold / 2):
+                self.writer.error(msg, path=name)
+            elif pc_cover < self.threshold:
+                self.writer.warning(msg, path=name)
+            else:
+                self.writer.info(msg, path=name)
         return percent
 
 if coverage is not None:
@@ -683,7 +720,7 @@ class PyCheckerChecker(BaseChecker):
     id = 'pychecker'
     need_preprocessor = 'install'
 
-    def _run(self, test):
+    def do_check(self, test):
         """run the checker against <path> (usually a directory)"""
         command = ['pychecker', '-Qqe', 'Style']
         command += get_module_files(pyinstall_path(test))
