@@ -1,9 +1,11 @@
 """APyCoT bot waiting for pyro connection
 
 :organization: Logilab
-:copyright: 2008-2009 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2008-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
+from __future__ import with_statement
+
 __docformat__ = "restructuredtext en"
 
 import os
@@ -17,7 +19,7 @@ from os.path import exists, join, normpath
 from subprocess import Popen
 from tempfile import TemporaryFile
 from threading import Thread, Lock, Timer
-
+from traceback import format_exc
 from datetime import datetime, timedelta
 
 from Pyro import config
@@ -31,14 +33,16 @@ config.PYRO_TRACEBACK_USER_LEVEL = 3
 config.PYRO_DETAILED_TRACEBACK = 1
 config.PYRO_STORAGE = '/tmp' # XXX
 
+from logilab.common import tasksqueue, pyro_ext as pyro
 from logilab.common.logging_ext import set_log_methods, init_log
 from logilab.common.configuration import Configuration, Method
-from logilab.common import tasksqueue, pyro_ext as pyro
+from logilab.common.textutils import get_csv
 
 from apycotbot import MODE, ConfigError, list_registered, get_registered
-from apycotbot.utils import (PYRO_OPTIONS, CW_OPTIONS, TEST_OPTIONS,
-                             RESOURCES_OPTIONS, ConnectionHandler)
-from apycotbot.task import ApycotTask, ARCHIVE_EXT, make_archive_name
+from apycotbot.utils import (
+    PYRO_OPTIONS, CW_OPTIONS, TEST_OPTIONS, RESOURCES_OPTIONS,
+    ConnectionHandler, connection_infos)
+from apycotbot.task import ApycotTask
 
 from apycotbot import checkers, preprocessors # trigger registration
 
@@ -48,13 +52,13 @@ class ApycotBotServer(Configuration):
         ('debug',
          {'action' : 'store_true', 'short': 'D',
           'help': 'start in debug mode',
-          'group': 'main', 'inputlevel': 2,
+          'group': 'main', 'level': 2,
           }),
         ('force',
          {'action' : 'store_true', 'short': 'f',
           'help': 'force server start even if already registered in the pyro \
 name server',
-          'group': 'main', 'inputlevel': 2,
+          'group': 'main', 'level': 2,
           }),
         ('generate-rcfile',
          {'action' : 'callback', 'callback' : Method('_generate_config'),
@@ -69,7 +73,7 @@ them in the generated configuration.'''}),
          {'type' : 'int', 'short': 'T',
           'default': 2,
           'help': 'number of tasks which may be run in parallel',
-          'group': 'process-control', 'inputlevel': 2,
+          'group': 'process-control', 'level': 2,
           }),
 
         ) + RESOURCES_OPTIONS + (
@@ -79,19 +83,19 @@ them in the generated configuration.'''}),
           'default': None,
           'help': 'if this option is set, use the specified user to start \
 the daemon rather than the user running the command',
-          'group': 'pyro-server', 'inputlevel': 1,
+          'group': 'pyro-server', 'level': 1,
           }),
         ('log-file',
          {'type' : 'string',
           'default': Method('_log_file'),
           'help': 'file where output logs should be written',
-          'group': 'pyro-server', 'inputlevel': 2,
+          'group': 'pyro-server', 'level': 2,
           }),
        ('log-threshold',
          {'type' : 'string', # XXX use a dedicated type?
           'default': 'INFO',
           'help': 'server\'s log level',
-          'group': 'pyro-server', 'inputlevel': 1,
+          'group': 'pyro-server', 'level': 1,
           }),
         ('host',
          {'type' : 'string',
@@ -99,13 +103,13 @@ the daemon rather than the user running the command',
           'help': 'host name if not correctly detectable through gethostname. '\
           'You can also specify a port using <host>:<port> notation (will be '\
           'choosen randomly if not set (recommended)).',
-          'group': 'pyro-server', 'inputlevel': 1,
+          'group': 'pyro-server', 'level': 1,
           }),
         ('pid-file',
          {'type' : 'string',
           'default': Method('_pid_file'),
           'help': 'daemon\'s pid file',
-          'group': 'pyro-server', 'inputlevel': 2,
+          'group': 'pyro-server', 'level': 2,
           }),
          ) + PYRO_OPTIONS
 
@@ -128,12 +132,14 @@ the daemon rather than the user running the command',
         # methods
         if self['plugins']:
             for module in self['plugins']:
-                writer.info('loading extra module %s', module)
+                self.info('loading extra module %s', module)
                 try:
                     __import__(module)
-                except Exception:
-                        writer.error('while loading %s', module, tb=True)
-                        continue
+                except Exception, ex:
+                    self.error('while loading %s: %s\n%s',
+                               module, ex, format_exc())
+                    continue
+        self._cnx_infos, self._aliases = connection_infos()
 
     if MODE == 'dev':
         _default_conf_file = os.path.expanduser('~/etc/apycotbot.ini')
@@ -247,43 +253,39 @@ the daemon rather than the user running the command',
         sys.stdout.close()
         sys.stderr.close()
 
+    def _cnxh(self, cwinstid):
+        cwinstid = cwinstid or self['cw-inst-id']
+        cwinstid = self._aliases.get(cwinstid, cwinstid)
+        return ConnectionHandler(cwinstid, self['pyro-ns-host'],
+                                 self._cnx_infos.get(cwinstid, {}))
+
     # public (pyro) interface #################################################
 
     def queue_all(self, branch=None, start_rev_deps=False, keep_test_dir=False,
-                  archive=False, priority='LOW', cwid=None):
+                  archive=False, priority='LOW', cwinstid=None):
         """add task to the queue for every activated test config"""
-        cnxh = ConnectionHandler(cwid or self['cw-inst-id'],
-                                 self['pyro-ns-host'])
-        cnxh.connect()
-        try:
+        with self._cnxh(cwinstid) as cnxh:
             for tconfig in cnxh.activated_tests().entities():
-                self._queue_tconfig(tconfig, priority, branch, start_rev_deps,
-                                    keep_test_dir, archive, cwid)
-        finally:
-            cnxh.close()
+                self._queue_tconfig(cnxh, tconfig, priority, branch, start_rev_deps,
+                                    keep_test_dir, archive)
 
     def queue_task(self, envname, tcname, branch=None, start_rev_deps=False,
                    keep_test_dir=False, archive=False, priority='LOW',
-                   cwid=None):
+                   cwinstid=None):
         """add a task to the queue for test config of the given name"""
-        cnxh = ConnectionHandler(cwid or self['cw-inst-id'],
-                                 self['pyro-ns-host'])
-        cnxh.connect()
-        try:
+        with self._cnxh(cwinstid) as cnxh:
             tconfig = cnxh.test_config(envname, tcname)
-            self._queue_tconfig(tconfig, priority, branch, start_rev_deps,
-                                keep_test_dir, archive, cwid)
-        finally:
-            cnxh.close()
+            self._queue_tconfig(cnxh, tconfig, priority, branch, start_rev_deps,
+                                keep_test_dir, archive)
 
-    def _queue_tconfig(self, tconfig, priority, branch, start_rev_deps,
-                       keep_test_dir, archive,  cwid):
+    def _queue_tconfig(self, cnxh, tconfig, priority, branch, start_rev_deps,
+                       keep_test_dir, archive):
         msg = 'queuing test %s (%s) with priority %s'
         args = (tconfig.name, tconfig.environment.name, priority)
         if not tconfig.all_checks:
             self.warning('no check defined for %s. Exit.' % tconfig.name)
             return
-        if self._already_running(tconfig, branch):
+        if self._already_running(cnxh, tconfig, branch):
             self.info('don\'t queue already running test %s for branch %s',
                       tconfig.name, branch)
             return
@@ -293,40 +295,48 @@ the daemon rather than the user running the command',
         self.info(msg, *args)
         from_inst_id = tconfig.metainformation()['source']['uri']
         if from_inst_id == 'system':
-            from_inst_id = cwid or self['cw-inst-id']
+            from_inst_id = cnxh.cwqinstid
+        else:
+            from_inst_id = self._aliases.get(from_inst_id, from_inst_id)
         self._queue.put(ApycotTask(tconfig.eid,
-                                   tconfig.environment.name, tconfig.name, from_inst_id,
+                                   tconfig.environment.name, tconfig.name,
+                                   from_inst_id,
                                    getattr(tasksqueue, priority),
                                    branch=branch,
                                    keep_test_dir=keep_test_dir,
-                                   archive=archive,
-                                   archive_dir=self["archive-dir"]))
+                                   archive=archive))
         if start_rev_deps:
             self.info('starting reverse dependencies')
             for pe in tconfig.environment.reverse_needs_checkout:
                 # pe may be a test config
                 for tc in getattr(pe, 'reverse_use_environment', ()):
                     if tc.state == 'activated' and tc.name == tconfig.name:
-                        self._queue_tconfig(tc, priority, branch,
+                        self._queue_tconfig(cnxh, tc, priority, branch,
                                             start_rev_deps=False,
                                             keep_test_dir=keep_test_dir,
-                                            archive=archive,
-                                            cwid=cwid)
+                                            archive=archive)
                     else:
                         self.info('skipped configuration %s (%s) for branch %s',
                                   tc.name, tc.state, branch)
 
-    def pending_tasks(self):
-        """return an ordered list of task pending in the queue, from the lowest
-        to the highest priority, and another list of currently running task
+    def pending_tasks(self, cwinstid=None):
+        """return an ordered list of tasks pending in the queue, from the lowest
+        to the highest priority, and another list of currently running tasks.
+
+        When an instance id is specified, the lists may contains some None
+        values corresponding to tasks from another instance.
         """
-        return ([t.as_dict() for t in self._queue],
-                [t.as_dict() for t in self._running_tasks])
+        if cwinstid is not None:
+            cwinstid = self._aliases.get(cwinstid, cwinstid)
+        def task_repr(t):
+            if cwinstid is None or t.cwinstid == cwinstid:
+                return t.as_dict()
+            return None
+        return ([task_repr(t) for t in self._queue],
+                [task_repr(t) for t in self._running_tasks])
 
     def cancel_task(self, tid):
-        """return an ordered list of task pending in the queue, from the highest
-        to the lowest priority
-        """
+        """remove task with the given identifier from the pending tasks queue"""
         self._queue.remove(tid)
         self.info('test %s cancelled', tid)
 
@@ -397,31 +407,6 @@ the daemon rather than the user running the command',
                                 })
         return res
 
-    def get_archive(self, instance_id, exec_id):
-        """get the content of the requested archive.
-
-        Return a String if the archive exist and readable, None otherwise.
-        """
-        exec_dir = normpath(self["archive-dir"])
-        archive_path = join(exec_dir, make_archive_name(instance_id, exec_id))
-        try:
-            return open(archive_path, mode='rb').read()
-        except IOError:
-            return None
-
-    def clear_archive(self, instance_id, exec_id):
-        """Remove an archive from pyro storage.
-
-        return nale of actually deleted archive"""
-        exec_dir = normpath(self["archive-dir"])
-        archive_name = make_archive_name(instance_id, exec_id)
-        archive_path = join(exec_dir, archive_name)
-        if exists(archive_path):
-            os.unlink(archive_path)
-            return archive_name
-
-
-
     # internal tasks managements ##############################################
 
     def _start_tasks(self):
@@ -482,12 +467,13 @@ the daemon rather than the user running the command',
             self.info('*************** %s std output:', pacname)
             self.info(outfile.read())
 
-    def _already_running(self, tc, branch):
+    def _already_running(self, cnxh, tc, branch):
         """before queuing a task, look at currently running tasks to avoid
         launching the same task twice in a short interval of time
         """
+        tid = '%s.%s' % (cnxh.cwqinstid, tc.eid)
         for task in self._running_tasks:
-            if task.id == tc.eid and task.branch == branch and \
+            if task.id == tid and task.branch == branch and \
                    datetime.now() - task.starttime < self._skip_duplicate_time_delta:
                 return True
         return False
