@@ -2,7 +2,7 @@
 """some common utilities shared by different APyCoT modules
 
 :organization: Logilab
-:copyright: 2003-2009 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 :license: General Public License version 2 - http://www.gnu.org/licenses
 """
@@ -37,7 +37,7 @@ PYRO_NS_OPTIONS = (
       'help': 'Pyro name server\'s host where the bot is registered. If not '\
       'set, will be detected by a broadcast query. You can also specify a '\
       'port using <host>:<port> notation.',
-      'group': 'pyro-name-server', 'inputlevel': 1,
+      'group': 'pyro-name-server', 'level': 1,
       }),
 )
 
@@ -46,7 +46,7 @@ PYRO_OPTIONS = (
      {'type' : 'string',
       'default': 'apycotbot',
       'help': 'identifier of the apycot bot in the pyro name-server.',
-      'group': 'pyro-server', 'inputlevel': 2,
+      'group': 'pyro-server', 'level': 2,
       }),
 ) + PYRO_NS_OPTIONS
 
@@ -56,7 +56,7 @@ CW_OPTIONS = (
       'help': 'identifier of the CubicWeb instance used to store tests '\
       'configuration and results in the pyro name-server.',
       'default': 'apycot',
-      'group': 'main', 'inputlevel': 0,
+      'group': 'main', 'level': 0,
       }),
 )
 
@@ -64,21 +64,15 @@ TEST_OPTIONS = (
     ('plugins',
      {'type' : 'csv',
       'help': 'comma separated list of plugins (eg python modules) that should be loaded at startup',
-      'group': 'main', 'inputlevel': 2,
+      'group': 'main', 'level': 2,
       }),
 
     ('test-dir',
      {'type' : 'file',
       'default': '/tmp',
       'help': 'directory where the test environment will be built',
-      'group': 'main', 'inputlevel': 2,
+      'group': 'main', 'level': 2,
       }),
-    ('archive-dir',
-     {'type' : 'file',
-      'default': '/tmp',
-      'help': 'directory where the test environment archive will be stored',
-      'group': 'main', 'inputlevel': 2,
-     }),
 )
 
 RESOURCES_OPTIONS = (
@@ -86,94 +80,126 @@ RESOURCES_OPTIONS = (
      {'type' : 'time',
       'default': None,
       'help': 'maximum CPU Time in second that may be used to execute a test.',
-      'group': 'process-control', 'inputlevel': 2,
+      'group': 'process-control', 'level': 2,
       }),
     ('max-time',
      {'type' : 'time',
       'default': None,
       'help': 'maximum Real Time in second that may be used to execute a test.',
-      'group': 'process-control', 'inputlevel': 2,
+      'group': 'process-control', 'level': 2,
       }),
     ('max-memory',
      {'type' : 'bytes',
       'default': None,
       'help': 'maximum Memory in bytes the test can allocate.',
-      'group': 'process-control', 'inputlevel': 2,
+      'group': 'process-control', 'level': 2,
       }),
     ('max-reprieve',
      {'type' : 'time',
       'default': 60,
       'help': 'delay in second while the test try to abort nicely (reporting '
       'the error and cleaning up the environement before it\'s killed).',
-      'group': 'process-control', 'inputlevel': 2,
+      'group': 'process-control', 'level': 2,
       }),
 )
 
 # cubicweb connection handling ################################################
 
+if MODE == 'dev':
+    _DEFAULT_SOURCES_FILE = os.path.expanduser('~/etc/apycotbot-cw-sources.ini')
+else:
+    _DEFAULT_SOURCES_FILE = '/etc/apycotbot-cw-sources.ini'
+_CW_SOURCES_FILE = os.environ.get('APYCOTBOTSOURCES', _DEFAULT_SOURCES_FILE)
+
+def connection_infos():
+    aliases = {}
+    if os.path.exists(_CW_SOURCES_FILE):
+        cnxinfos = read_config(_CW_SOURCES_FILE)
+        for cwinstid, instinfo in cnxinfos.items():
+            nsgroup, instid = ns_group_and_id(cwinstid, defaultnsgroup=None)
+            if instinfo.get('pyro-ns-group', nsgroup) != nsgroup:
+                raise ConfigError('conflicting values for pyro-ns-group')
+            nsgroup = instinfo.get('pyro-ns-group', nsgroup or 'cubicweb')
+            qinstid = ':%s.%s' % (nsgroup, instid)
+            if 'aliases' in instinfo:
+                for alias in get_csv(instinfo['aliases']):
+                    aliasnsgroup, aliasinstid = ns_group_and_id(
+                        alias, defaultnsgroup=nsgroup)
+                    qalias = ':%s.%s' % (aliasnsgroup, aliasinstid)
+                    assert qalias not in aliases, 'duplicated alias %s' % qalias
+                    aliases[qalias] = qinstid
+            if cwinstid != qinstid:
+                cnxinfos[qinstid] = cnxinfos.pop(cwinstid)
+    else:
+        cnxinfos = {}
+    return cnxinfos, aliases
+
+
 class ConnectionHandler(object):
     """handle connection to a cubicweb repository"""
 
-    if MODE == 'dev':
-        _default_sources_file = os.path.expanduser('~/etc/apycotbot-cw-sources.ini')
-    else:
-        _default_sources_file = '/etc/apycotbot-cw-sources.ini'
-
-    def __init__(self, cwinstid, nshost=None):
+    def __init__(self, cwinstid, nshost=None, cnxinfo=None):
         if not cwinstid:
             raise ConfigError('you must specify main cubicweb instance '
                               'identifier in the configuration file or using '
                               'the --cw-inst-id option')
+        if cnxinfo is None:
+            cnxinfo = connection_infos()[0].get(cwinstid, {})
         nsgroup, instid = ns_group_and_id(cwinstid, defaultnsgroup=None)
-        self.cw_pyro_instid = instid
-        if os.path.exists(self._cw_sources_file):
-            self._cnx_infos = read_config(self._cw_sources_file)
-        else:
-            self._cnx_infos = {}
-        system_info = self._cnx_infos.setdefault(instid, {})
+        self.cwinstid = instid
+        self.cnxinfo = cnxinfo
         if nsgroup:
-            if not system_info.get('pyro-ns-group', nsgroup) == nsgroup:
+            if cnxinfo.get('pyro-ns-group', nsgroup) != nsgroup:
                 raise ConfigError('conflicting values for pyro-ns-group')
-            system_info['pyro-ns-group'] = nsgroup
-        if nshost and not 'pyro-ns-host' in system_info:
-            system_info['pyro-ns-host'] = nshost
-        self.cnx = None
+            cnxinfo['pyro-ns-group'] = nsgroup
+        else:
+            nsgroup = ':cubicweb'
+        self.cwqinstid = '%s.%s' % (nsgroup, self.cwinstid)
+        if nshost and not 'pyro-ns-host' in cnxinfo:
+            cnxinfo['pyro-ns-host'] = nshost
+        self.cnx = self.cw = None
         self._cu = None
 
-    @property
-    def _cw_sources_file(self):
-        return os.environ.get('APYCOTBOTSOURCES', self._default_sources_file)
+    def __enter__(self):
+        self.connect()
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
 
     def connect(self):
-        mycnxinfo = self._cnx_infos[self.cw_pyro_instid]
-        user = mycnxinfo.get('user', 'apycotbot')
-        password = mycnxinfo.get('password', 'apycot')
-        group = mycnxinfo.get('pyro-ns-group', None)
-        host = mycnxinfo.get('pyro-ns-host', None)
+        user = self.cnxinfo.get('user', 'apycotbot')
+        password = self.cnxinfo.get('password', 'apycot')
+        group = self.cnxinfo.get('pyro-ns-group', None)
+        host = self.cnxinfo.get('pyro-ns-host', None)
         try:
-            self.cnx = connect(self.cw_pyro_instid, user, password,
-                                group=group, host=host, initlog=False)
-            self._cu = self.cnx.cursor()
+            self.cnx = connect(database=self.cwinstid, login=user,
+                               password=password, group=group, host=host,
+                               initlog=False)
         except Exception, ex:
             self.critical('error while trying to connect to instance %r: %s',
-                          self.cw_pyro_instid, ex)
+                          self.cwinstid, ex)
             return None
         else:
             self.cnx.load_appobjects(cubes='apycot', subpath=('entities',),
                                      expand=False)
+            # don't get the cursor until load_appobjects has been called
+            self._cu = self.cnx.cursor()
+            self.cw = self._cu.req
             return self.cnx
 
     def activated_tests(self):
         if self.cnx is None and not self.connect():
-            raise Exception('can\'t list activated tests, can\'t connect to %s'
-                            % (self.cw_pyro_instid))
+            raise Exception('can\'t list activated tests, apycotbot can\'t connect to %s. '
+                            'Check apycotbot logs for details.' % (self.cwinstid))
         return self.execute(
             'Any X WHERE X is TestConfig, X in_state S, S name "activated"')
 
     def test_config(self, pename, tcname, reconnect=True):
         if self.cnx is None and not self.connect():
-            raise Exception('can\'t start test %s, can\'t connect to %s'
-                            % (tcname, self.cw_pyro_instid))
+            raise Exception('can\'t start test %s, apycotbot can\'t connect to '
+                            '%s. Check apycotbot logs for details.'
+                            % (tcname, self.cwinstid))
         try:
             # check test is known
             return self.execute('Any PE, PEN, TC, TCN WHERE TC use_environment PE, '
@@ -185,12 +211,12 @@ class ConnectionHandler(object):
         except IndexError:
             raise Exception('unknown test %s.%s' % (pename, tcname))
 
-    def execute(self, rql, kwargs=None, cachekeys=None, reconnect=True):
+    def execute(self, rql, kwargs=None, reconnect=True):
         try:
-            return self._cu.execute(rql, kwargs, cachekeys)
+            return self._cu.execute(rql, kwargs)
         except (BadConnectionId, errors.PyroError):
             if reconnect and self.connect():
-                return self.execute(rql, kwargs, cachekeys, reconnect=False)
+                return self.execute(rql, kwargs, reconnect=False)
             raise
 
     def commit(self):
@@ -263,6 +289,10 @@ class ApycotObject(object):
     def __init__(self, writer, options=None):
         self.writer = writer
         self.options = options or {}
+        for optdict in self.options_def:
+            assert hasattr(optdict, 'get'), "optdict : %s ; self.options : %s" % (optdict, self.options)
+            if 'default' in optdict and optdict['name'] not in self.options:
+                self.options[optdict['name']] = optdict['default']
 
     @staticmethod
     def merge_status(global_status, status):
@@ -440,7 +470,7 @@ class Command(ApycotObject):
         self.parsed_content = parsed_content
         self.raises = raises
         self.shell = shell
-        self.path = None
+        self.path = path
         self._cmd_printed = False
 
     @property
@@ -477,23 +507,22 @@ class Command(ApycotObject):
         stdout, stderr, unparsed = self.process_output(stdout, stderr)
         cmd = self.commandstr
         path = self.path or cmd
-        print 'handle output', cmd
         if status:
             if status > 0:
-                msg = u'`%s` returned with status : %s' % (cmd, status)
+                short_msg = u'`%s` returned with status : %s' % (cmd, status)
                 cmd_status = self.non_zero_status_code
             else:
                 # negative status mean the process have been killed by a signal
-                msg = u'`%s` killed by signal %s' % (cmd, status)
+                short_msg = u'`%s` killed by signal %s' % (cmd, status)
                 # Get the signal number
                 status *= -1
                 cmd_status = KILLED
                 # XXX we need detection of common limit here
-            if self.raises:
-                raise SetupException(msg)
-            msg = self.append_output_messages(msg, stdout, stderr, unparsed)
+            msg = self.append_output_messages(short_msg, stdout, stderr, unparsed)
             self.writer.error(msg, path=path)
             self.set_status(cmd_status)
+            if self.raises:
+                raise SetupException(short_msg)
         else:
             msg = self.append_output_messages(u'`%s` executed successfuly' % cmd,
                                               stdout, stderr, unparsed)
